@@ -4,34 +4,35 @@
 Note that this module is named 'built_ins' so as not to be confused with the
 special Python builtins module.
 """
+import atexit
+import builtins
+from collections import Sequence
+from contextlib import contextmanager
+import inspect
+from glob import iglob
 import os
 import re
-import sys
-import time
 import shlex
-import atexit
 import signal
-import inspect
-import builtins
-import tempfile
-from glob import glob, iglob
 from subprocess import Popen, PIPE, STDOUT, CalledProcessError
-from contextlib import contextmanager
-from collections import Sequence, MutableMapping, Iterable
+import sys
+import tempfile
+import time
 
-from xonsh.tools import (
-    suggest_commands, XonshError, ON_POSIX, ON_WINDOWS, string_types,
-    expandvars, CommandsCache
-)
-from xonsh.inspectors import Inspector
+from xonsh.aliases import Aliases, make_default_aliases
 from xonsh.environ import Env, default_env, locate_binary
-from xonsh.aliases import DEFAULT_ALIASES
+from xonsh.foreign_shells import load_foreign_aliases
+from xonsh.history import History
+from xonsh.inspectors import Inspector
 from xonsh.jobs import add_job, wait_for_active_job
+from xonsh.platform import ON_POSIX, ON_WINDOWS
 from xonsh.proc import (ProcProxy, SimpleProcProxy, ForegroundProcProxy,
                         SimpleForegroundProcProxy, TeePTYProc,
                         CompletedCommand, HiddenCompletedCommand)
-from xonsh.history import History
-from xonsh.foreign_shells import load_foreign_aliases
+from xonsh.tools import (
+    suggest_commands, XonshError, expandvars, CommandsCache
+)
+
 
 ENV = None
 BUILTINS_LOADED = False
@@ -56,118 +57,6 @@ def resetting_signal_handle(sig, f):
     signal.signal(sig, newh)
 
 
-class Aliases(MutableMapping):
-    """Represents a location to hold and look up aliases."""
-
-    def __init__(self, *args, **kwargs):
-        self._raw = {}
-        self.update(*args, **kwargs)
-
-    def get(self, key, default=None):
-        """Returns the (possibly modified) value. If the key is not present,
-        then `default` is returned.
-        If the value is callable, it is returned without modification. If it
-        is an iterable of strings it will be evaluated recursively to expand
-        other aliases, resulting in a new list or a "partially applied"
-        callable.
-        """
-        val = self._raw.get(key)
-        if val is None:
-            return default
-        elif isinstance(val, Iterable) or callable(val):
-            return self.eval_alias(val, seen_tokens={key})
-        else:
-            msg = 'alias of {!r} has an inappropriate type: {!r}'
-            raise TypeError(msg.format(key, val))
-
-    def eval_alias(self, value, seen_tokens, acc_args=[]):
-        """
-        "Evaluates" the alias `value`, by recursively looking up the leftmost
-        token and "expanding" if it's also an alias.
-
-        A value like ["cmd", "arg"] might transform like this:
-        > ["cmd", "arg"] -> ["ls", "-al", "arg"] -> callable()
-        where `cmd=ls -al` and `ls` is an alias with its value being a
-        callable.  The resulting callable will be "partially applied" with
-        ["-al", "arg"].
-        """
-        # Beware of mutability: default values for keyword args are evaluated
-        # only once.
-        if callable(value):
-            if acc_args:  # Partial application
-                def _alias(args, stdin=None):
-                    args = [expand_path(i) for i in (acc_args + args)]
-                    return value(args, stdin=stdin)
-                return _alias
-            else:
-                return value
-        else:
-            token, *rest = value
-            if token in seen_tokens or token not in self._raw:
-                # ^ Making sure things like `egrep=egrep --color=auto` works,
-                # and that `l` evals to `ls --color=auto -CF` if `l=ls -CF`
-                # and `ls=ls --color=auto`
-                return value + acc_args
-            else:
-                return self.eval_alias(self._raw[token],
-                                       seen_tokens | {token},
-                                       rest + acc_args)
-
-    def expand_alias(self, line):
-        """Expands any aliases present in line if alias does not point to a
-        builtin function and if alias is only a single command.
-        """
-        word = line.split(' ', 1)[0]
-        if word in builtins.aliases and isinstance(self.get(word), Sequence):
-            word_idx = line.find(word)
-            expansion = ' '.join(self.get(word))
-            line = line[:word_idx] + expansion + line[word_idx+len(word):]
-        return line
-
-    #
-    # Mutable mapping interface
-    #
-
-    def __getitem__(self, key):
-        return self._raw[key]
-
-    def __setitem__(self, key, val):
-        if isinstance(val, string_types):
-            self._raw[key] = shlex.split(val)
-        else:
-            self._raw[key] = val
-
-    def __delitem__(self, key):
-        del self._raw[key]
-
-    def update(self, *args, **kwargs):
-        for key, val in dict(*args, **kwargs).items():
-            self[key] = val
-
-    def __iter__(self):
-        yield from self._raw
-
-    def __len__(self):
-        return len(self._raw)
-
-    def __str__(self):
-        return str(self._raw)
-
-    def __repr__(self):
-        return '{0}.{1}({2})'.format(self.__class__.__module__,
-                                     self.__class__.__name__, self._raw)
-
-    def _repr_pretty_(self, p, cycle):
-        name = '{0}.{1}'.format(self.__class__.__module__,
-                                self.__class__.__name__)
-        with p.group(0, name + '(', ')'):
-            if cycle:
-                p.text('...')
-            elif len(self):
-                p.break_()
-                p.pretty(dict(self))
-
-
 def helper(x, name=''):
     """Prints help about, and then returns that variable."""
     INSPECTOR.pinfo(x, oname=name, detail_level=0)
@@ -267,30 +156,38 @@ def reglob(path, parts=None, i=None):
     return paths
 
 
-def regexpath(s):
+def regexpath(s, pymode=False):
     """Takes a regular expression string and returns a list of file
     paths that match the regex.
     """
     s = expand_path(s)
     o = reglob(s)
-    return o if len(o) != 0 else [s]
+    no_match = [] if pymode else [s]
+    return o if len(o) != 0 else no_match
 
 
 def globpath(s, ignore_case=False):
     """Simple wrapper around glob that also expands home and env vars."""
-    s = expand_path(s)
-    if ignore_case:
-        s = expand_case_matching(s)
-    o = glob(s)
+    o, s = _iglobpath(s, ignore_case=ignore_case)
+    o = list(o)
     return o if len(o) != 0 else [s]
 
 
-def iglobpath(s, ignore_case=False):
-    """Simple wrapper around iglob that also expands home and env vars."""
+def _iglobpath(s, ignore_case=False):
     s = expand_path(s)
     if ignore_case:
         s = expand_case_matching(s)
-    return iglob(s)
+    if sys.version_info > (3, 5):
+        if '**' in s and '**/*' not in s:
+            s = s.replace('**', '**/*')
+        # `recursive` is only a 3.5+ kwarg.
+        return iglob(s, recursive=True), s
+    else:
+        return iglob(s), s
+
+def iglobpath(s, ignore_case=False):
+    """Simple wrapper around iglob that also expands home and env vars."""
+    return _iglobpath(s, ignore_case)[0]
 
 
 RE_SHEBANG = re.compile(r'#![ \t]*(.+?)$')
@@ -400,8 +297,12 @@ def _open(fname, mode):
         return fname
     try:
         return open(fname, mode)
-    except Exception:
+    except PermissionError:
+        raise XonshError('xonsh: {0}: permission denied'.format(fname))
+    except FileNotFoundError:
         raise XonshError('xonsh: {0}: no such file or directory'.format(fname))
+    except Exception:
+        raise XonshError('xonsh: {0}: unable to open file'.format(fname))
 
 
 def _redirect_io(streams, r, loc=None):
@@ -501,7 +402,7 @@ def run_subproc(cmds, captured=False):
         procinfo['args'] = list(cmd)
         stdin = None
         stderr = None
-        if isinstance(cmd, string_types):
+        if isinstance(cmd, str):
             continue
         streams = {}
         while True:
@@ -525,13 +426,18 @@ def run_subproc(cmds, captured=False):
         elif prev_proc is not None:
             stdin = prev_proc.stdout
         # set standard output
+        _stdout_name = None
+        _stderr_name = None
         if 'stdout' in streams:
             if ix != last_cmd:
                 raise XonshError('Multiple redirects for stdout')
             stdout = streams['stdout'][-1]
             procinfo['stdout_redirect'] = streams['stdout'][:-1]
-        elif _capture_streams or ix != last_cmd:
+        elif ix != last_cmd:
             stdout = PIPE
+        elif _capture_streams:
+            _nstdout = stdout = tempfile.NamedTemporaryFile(delete=False)
+            _stdout_name = stdout.name
         elif builtins.__xonsh_stdout_uncaptured__ is not None:
             stdout = builtins.__xonsh_stdout_uncaptured__
         else:
@@ -540,8 +446,9 @@ def run_subproc(cmds, captured=False):
         if 'stderr' in streams:
             stderr = streams['stderr'][-1]
             procinfo['stderr_redirect'] = streams['stderr'][:-1]
-        elif captured == 'object':
-            stderr = PIPE
+        elif captured == 'object' and ix == last_cmd:
+            _nstderr = stderr = tempfile.NamedTemporaryFile(delete=False)
+            _stderr_name = stderr.name
         elif builtins.__xonsh_stderr_uncaptured__ is not None:
             stderr = builtins.__xonsh_stderr_uncaptured__
         uninew = (ix == last_cmd) and (not _capture_streams)
@@ -656,7 +563,15 @@ def run_subproc(cmds, captured=False):
     # get output
     output = b''
     if write_target is None:
-        if prev_proc.stdout not in (None, sys.stdout):
+        if _stdout_name is not None:
+            with open(_stdout_name, 'rb') as stdoutfile:
+                output = stdoutfile.read()
+            try:
+                _nstdout.close()
+            except:
+                pass
+            os.unlink(_stdout_name)
+        elif prev_proc.stdout not in (None, sys.stdout):
             output = prev_proc.stdout.read()
         if _capture_streams:
             # to get proper encoding from Popen, we have to
@@ -666,13 +581,24 @@ def run_subproc(cmds, captured=False):
             output = output.replace('\r\n', '\n')
         else:
             hist.last_cmd_out = output
-        if (captured == 'object' and
-                prev_proc.stderr not in {None, sys.stderr}):
-            errout = prev_proc.stderr.read()
-            errout = errout.decode(encoding=ENV.get('XONSH_ENCODING'),
-                                   errors=ENV.get('XONSH_ENCODING_ERRORS'))
-            errout = errout.replace('\r\n', '\n')
-            procinfo['stderr'] = errout
+        if captured == 'object': # get stderr as well
+            named = _stderr_name is not None
+            unnamed = prev_proc.stderr not in {None, sys.stderr}
+            if named:
+                with open(_stderr_name, 'rb') as stderrfile:
+                    errout = stderrfile.read()
+                try:
+                    _nstderr.close()
+                except:
+                    pass
+                os.unlink(_stderr_name)
+            elif unnamed:
+                errout = prev_proc.stderr.read()
+            if named or unnamed:
+                errout = errout.decode(encoding=ENV.get('XONSH_ENCODING'),
+                                       errors=ENV.get('XONSH_ENCODING_ERRORS'))
+                errout = errout.replace('\r\n', '\n')
+                procinfo['stderr'] = errout
 
     if (not prev_is_proxy and
             hist.last_cmd_rtn is not None and
@@ -728,23 +654,24 @@ def subproc_uncaptured(*cmds):
 
 def ensure_list_of_strs(x):
     """Ensures that x is a list of strings."""
-    if isinstance(x, string_types):
+    if isinstance(x, str):
         rtn = [x]
     elif isinstance(x, Sequence):
-        rtn = [i if isinstance(i, string_types) else str(i) for i in x]
+        rtn = [i if isinstance(i, str) else str(i) for i in x]
     else:
         rtn = [str(x)]
     return rtn
 
 
-def load_builtins(execer=None, config=None):
+def load_builtins(execer=None, config=None, login=False, ctx=None):
     """Loads the xonsh builtins into the Python builtins. Sets the
     BUILTINS_LOADED variable to True.
     """
     global BUILTINS_LOADED, ENV
     # private built-ins
-    builtins.__xonsh_env__ = ENV = Env(default_env(config=config))
-    builtins.__xonsh_ctx__ = {}
+    builtins.__xonsh_config__ = {}
+    builtins.__xonsh_env__ = ENV = Env(default_env(config=config, login=login))
+    builtins.__xonsh_ctx__ = {} if ctx is None else ctx
     builtins.__xonsh_help__ = helper
     builtins.__xonsh_superhelp__ = superhelper
     builtins.__xonsh_regexpath__ = regexpath
@@ -772,8 +699,11 @@ def load_builtins(execer=None, config=None):
     builtins.evalx = None if execer is None else execer.eval
     builtins.execx = None if execer is None else execer.exec
     builtins.compilex = None if execer is None else execer.compile
-    builtins.default_aliases = builtins.aliases = Aliases(DEFAULT_ALIASES)
-    builtins.aliases.update(load_foreign_aliases(issue_warning=False))
+
+    # Need this inline/lazy import here since we use locate_binary that relies on __xonsh_env__ in default aliases
+    builtins.default_aliases = builtins.aliases = Aliases(make_default_aliases())
+    if login:
+        builtins.aliases.update(load_foreign_aliases(issue_warning=False))
     # history needs to be started after env and aliases
     # would be nice to actually include non-detyped versions.
     builtins.__xonsh_history__ = History(env=ENV.detype(),
@@ -802,7 +732,8 @@ def unload_builtins():
         builtins.quit = builtins.__xonsh_pyquit__
     if not BUILTINS_LOADED:
         return
-    names = ['__xonsh_env__',
+    names = ['__xonsh_config__',
+             '__xonsh_env__',
              '__xonsh_ctx__',
              '__xonsh_help__',
              '__xonsh_superhelp__',
