@@ -1,32 +1,37 @@
 # -*- coding: utf-8 -*-
 """Environment for the xonsh shell."""
-import os
-import re
+import builtins
+from collections import Mapping, MutableMapping, MutableSequence, MutableSet, namedtuple
+from contextlib import contextmanager
+from functools import wraps
+from itertools import chain
 import json
+import locale
+import os
+from pprint import pformat
+import re
 import socket
 import string
-import locale
-import builtins
 import subprocess
-from itertools import chain
+import sys
 from warnings import warn
-from pprint import pformat
-from functools import wraps
-from contextlib import contextmanager
-from collections import MutableMapping, MutableSequence, MutableSet, namedtuple
 
 from xonsh import __version__ as XONSH_VERSION
-from xonsh.tools import (
-    ON_WINDOWS, ON_MAC, ON_LINUX, ON_ARCH, IS_ROOT,
-    always_true, always_false, ensure_string, is_env_path, str_to_env_path,
-    env_path_to_str, is_bool, to_bool, bool_to_str, is_history_tuple, to_history_tuple,
-    history_tuple_to_str, is_float, string_types, is_string, DEFAULT_ENCODING,
-    is_completions_display_value, to_completions_display_value, is_string_set,
-    csv_to_set, set_to_csv, get_sep, is_int, is_bool_seq, csv_to_bool_seq,
-    bool_seq_to_csv, DefaultNotGiven, setup_win_unicode_console
-)
+from xonsh.codecache import run_script_with_cache
 from xonsh.dirstack import _get_cwd
 from xonsh.foreign_shells import DEFAULT_SHELLS, load_foreign_envs
+from xonsh.platform import (BASH_COMPLETIONS_DEFAULT, ON_ANACONDA, ON_LINUX,
+                            ON_WINDOWS, DEFAULT_ENCODING)
+from xonsh.tools import (
+    IS_SUPERUSER, always_true, always_false, ensure_string, is_env_path,
+    str_to_env_path, env_path_to_str, is_bool, to_bool, bool_to_str,
+    is_history_tuple, to_history_tuple, history_tuple_to_str, is_float,
+    is_string, is_completions_display_value, to_completions_display_value,
+    is_string_set, csv_to_set, set_to_csv, get_sep, is_int, is_bool_seq,
+    csv_to_bool_seq, bool_seq_to_csv, DefaultNotGiven, print_exception,
+    setup_win_unicode_console, intensify_colors_on_win_setter
+)
+
 
 LOCALE_CATS = {
     'LC_CTYPE': locale.LC_CTYPE,
@@ -67,6 +72,7 @@ DEFAULT_ENSURERS = {
     'FORCE_POSIX_PATHS': (is_bool, to_bool, bool_to_str),
     'HISTCONTROL': (is_string_set, csv_to_set, set_to_csv),
     'IGNOREEOF': (is_bool, to_bool, bool_to_str),
+    'INTENSIFY_COLORS_ON_WIN':(always_false, intensify_colors_on_win_setter, bool_to_str),
     'LC_COLLATE': (always_false, locale_convert('LC_COLLATE'), ensure_string),
     'LC_CTYPE': (always_false, locale_convert('LC_CTYPE'), ensure_string),
     'LC_MESSAGES': (always_false, locale_convert('LC_MESSAGES'), ensure_string),
@@ -81,7 +87,10 @@ DEFAULT_ENSURERS = {
     'RAISE_SUBPROC_ERROR': (is_bool, to_bool, bool_to_str),
     'RIGHT_PROMPT': (is_string, ensure_string, ensure_string),
     'TEEPTY_PIPE_DELAY': (is_float, float, str),
+    'UPDATE_OS_ENVIRON': (is_bool, to_bool, bool_to_str),
     'XONSHRC': (is_env_path, str_to_env_path, env_path_to_str),
+    'XONSH_CACHE_SCRIPTS': (is_bool, to_bool, bool_to_str),
+    'XONSH_CACHE_EVERYTHING': (is_bool, to_bool, bool_to_str),
     'XONSH_COLOR_STYLE': (is_string, ensure_string, ensure_string),
     'XONSH_ENCODING': (is_string, ensure_string, ensure_string),
     'XONSH_ENCODING_ERRORS': (is_string, ensure_string, ensure_string),
@@ -149,14 +158,7 @@ DEFAULT_VALUES = {
     'AUTO_CD': False,
     'AUTO_PUSHD': False,
     'AUTO_SUGGEST': True,
-    'BASH_COMPLETIONS': (('/usr/local/etc/bash_completion',
-                             '/opt/local/etc/profile.d/bash_completion.sh')
-                        if ON_MAC else
-                        ('/usr/share/bash-completion/bash_completion',
-                             '/usr/share/bash-completion/completions/git')
-                        if ON_ARCH else
-                        ('/etc/bash_completion',
-                             '/usr/share/bash-completion/completions/git')),
+    'BASH_COMPLETIONS': BASH_COMPLETIONS_DEFAULT,
     'CASE_SENSITIVE_COMPLETIONS': ON_LINUX,
     'CDPATH': (),
     'COMPLETIONS_DISPLAY': 'multi',
@@ -167,6 +169,7 @@ DEFAULT_VALUES = {
     'HISTCONTROL': set(),
     'IGNOREEOF': False,
     'INDENT': '    ',
+    'INTENSIFY_COLORS_ON_WIN': True,
     'LC_CTYPE': locale.setlocale(locale.LC_CTYPE),
     'LC_COLLATE': locale.setlocale(locale.LC_COLLATE),
     'LC_TIME': locale.setlocale(locale.LC_TIME),
@@ -179,8 +182,6 @@ DEFAULT_VALUES = {
     'PATH': (),
     'PATHEXT': (),
     'PROMPT': DEFAULT_PROMPT,
-    'PROMPT_TOOLKIT_COLORS': {},
-    'PROMPT_TOOLKIT_STYLES': None,
     'PUSHD_MINUS': False,
     'PUSHD_SILENT': False,
     'RAISE_SUBPROC_ERROR': False,
@@ -191,6 +192,7 @@ DEFAULT_VALUES = {
     'SUGGEST_THRESHOLD': 3,
     'TEEPTY_PIPE_DELAY': 0.01,
     'TITLE': DEFAULT_TITLE,
+    'UPDATE_OS_ENVIRON': False,
     'VI_MODE': False,
     'WIN_UNICODE_CONSOLE': True,
     'XDG_CONFIG_HOME': os.path.expanduser(os.path.join('~', '.config')),
@@ -200,6 +202,8 @@ DEFAULT_VALUES = {
                               'xonsh', 'xonshrc'),
                 os.path.expanduser('~/.xonshrc')) if ON_WINDOWS
                else ('/etc/xonshrc', os.path.expanduser('~/.xonshrc'))),
+    'XONSH_CACHE_SCRIPTS': True,
+    'XONSH_CACHE_EVERYTHING': False,
     'XONSH_COLOR_STYLE': 'default',
     'XONSH_CONFIG_DIR': xonsh_config_dir,
     'XONSH_DATA_DIR': xonsh_data_dir,
@@ -307,6 +311,11 @@ DEFAULT_DOCS = {
         "exit status) to not be added to the history list."),
     'IGNOREEOF': VarDocs('Prevents Ctrl-D from exiting the shell.'),
     'INDENT': VarDocs('Indentation string for multiline input'),
+    'INTENSIFY_COLORS_ON_WIN': VarDocs('Enhance style colors for readability '
+        'when using the default terminal (cmd.exe) on winodws. Blue colors, '
+        'which are hard to read, are replaced with cyan. Other colors are '
+        'generally replaced by their bright counter parts.',
+        configurable=ON_WINDOWS),
     'LOADED_CONFIG': VarDocs('Whether or not the xonsh config file was loaded',
         configurable=False),
     'LOADED_RC_FILES': VarDocs(
@@ -333,17 +342,6 @@ DEFAULT_DOCS = {
         "auto-formatted, see 'Customizing the Prompt' at "
         'http://xon.sh/tutorial.html#customizing-the-prompt.',
         default='xonsh.environ.DEFAULT_PROMPT'),
-    'PROMPT_TOOLKIT_COLORS': VarDocs(
-        'This is a mapping of from color names to HTML color codes. Whenever '
-        'prompt-toolkit would color a word a particular color (in the prompt, '
-        'or in syntax highlighting), it will use the value specified here to '
-        'represent that color, instead of its default.  If a color is not '
-        'specified here, prompt-toolkit uses the colors from '
-        "'xonsh.tools._PT_COLORS'.", configurable=False),
-    'PROMPT_TOOLKIT_STYLES': VarDocs(
-        'This is a mapping of pygments tokens to user-specified styles for '
-        'prompt-toolkit. See the prompt-toolkit and pygments documentation '
-        'for more details. If None, this is skipped.', configurable=False),
     'PUSHD_MINUS': VarDocs(
         'Flag for directory pushing functionality. False is the normal '
         'behavior.'),
@@ -351,9 +349,9 @@ DEFAULT_DOCS = {
         'Whether or not to suppress directory stack manipulation output.'),
     'RAISE_SUBPROC_ERROR': VarDocs(
         'Whether or not to raise an error if a subprocess (captured or '
-        'uncaptured) returns a non-zero exit status, which indicates failure.'
+        'uncaptured) returns a non-zero exit status, which indicates failure. '
         'This is most useful in xonsh scripts or modules where failures '
-        'should cause an end to execution. This is less useful at a terminal.'
+        'should cause an end to execution. This is less useful at a terminal. '
         'The error that is raised is a subprocess.CalledProcessError.'),
     'RIGHT_PROMPT': VarDocs('Template string for right-aligned text '
         'at the prompt. This may be parameterized in the same way as '
@@ -388,16 +386,30 @@ DEFAULT_DOCS = {
         'used. This can be used to fix situations where a spawned process, '
         'such as piping into \'grep\', exits too quickly for the piping '
         'operation itself. TeePTY (and thus this variable) are currently '
-        'only used when $XONSH_STORE_STDOUT is True.', configurable=ON_LINUX),
+        'only used when $XONSH_STORE_STDOUT is True.',
+        configurable=ON_LINUX),
     'TERM': VarDocs(
         'TERM is sometimes set by the terminal emulator. This is used (when '
         "valid) to determine whether or not to set the title. Users shouldn't "
-        "need to set this themselves.", configurable=False),
+        "need to set this themselves. Note that this variable should be set as "
+        "early as possible in order to ensure it is effective. Here are a few "
+        "options:\n\n"
+        "* Set this from the program that launches xonsh. On posix systems, \n"
+        "  this can be performed by using env, e.g. \n"
+        "  '/usr/bin/env TERM=xterm-color xonsh' or similar.\n"
+        "* From the xonsh command line, namely 'xonsh -DTERM=xterm-color'.\n"
+        "* In the config file with '{\"env\": {\"TERM\": \"xterm-color\"}}'.\n"
+        "* Lastly, in xonshrc with '$TERM'\n\n"
+        "Ideally, your terminal emulator will set this correctly but that does "
+        "not always happen.", configurable=False),
     'TITLE': VarDocs(
         'The title text for the window in which xonsh is running. Formatted '
         "in the same manner as $PROMPT, see 'Customizing the Prompt' "
         'http://xon.sh/tutorial.html#customizing-the-prompt.',
         default='xonsh.environ.DEFAULT_TITLE'),
+    'UPDATE_OS_ENVIRON': VarDocs("If True os.environ will always be updated "
+        "when the xonsh environment changes. The environment can be reset to "
+        "the default value by calling '__xonsh_env__.undo_replace_env()'"),
     'VI_MODE': VarDocs(
         "Flag to enable 'vi_mode' in the 'prompt_toolkit' shell."),
     'VIRTUAL_ENV': VarDocs(
@@ -423,6 +435,12 @@ DEFAULT_DOCS = {
         'control file if there is a naming collision.', default=(
         "On Linux & Mac OSX: ('/etc/xonshrc', '~/.xonshrc')\n"
         "On Windows: ('%ALLUSERSPROFILE%\\xonsh\\xonshrc', '~/.xonshrc')")),
+    'XONSH_CACHE_SCRIPTS': VarDocs(
+        'Controls whether the code for scripts run from xonsh will be cached'
+        ' (``True``) or re-compiled each time (``False``).'),
+    'XONSH_CACHE_EVERYTHING': VarDocs(
+        'Controls whether all code (including code enetered at the interactive'
+        ' prompt) will be cached.'),
     'XONSH_COLOR_STYLE': VarDocs(
         'Sets the color style for xonsh colors. This is a style name, not '
         'a color map.'),
@@ -495,6 +513,7 @@ class Env(MutableMapping):
     def __init__(self, *args, **kwargs):
         """If no initial environment is given, os.environ is used."""
         self._d = {}
+        self._orig_env = None
         self.ensurers = {k: Ensurer(*v) for k, v in DEFAULT_ENSURERS.items()}
         self.defaults = DEFAULT_VALUES
         self.docs = DEFAULT_DOCS
@@ -503,16 +522,19 @@ class Env(MutableMapping):
         for key, val in dict(*args, **kwargs).items():
             self[key] = val
         self._detyped = None
-        self._orig_env = None
+
+    @staticmethod
+    def detypeable(val):
+        return not (callable(val) or isinstance(val, MutableMapping))
 
     def detype(self):
         if self._detyped is not None:
             return self._detyped
         ctx = {}
         for key, val in self._d.items():
-            if callable(val) or isinstance(val, MutableMapping):
+            if not self.detypeable(val):
                 continue
-            if not isinstance(key, string_types):
+            if not isinstance(key, str):
                 key = str(key)
             ensurer = self.get_ensurer(key)
             val = ensurer.detype(val)
@@ -544,16 +566,14 @@ class Env(MutableMapping):
         if key in self.ensurers:
             return self.ensurers[key]
         for k, ensurer in self.ensurers.items():
-            if isinstance(k, string_types):
+            if isinstance(k, str):
                 continue
-            m = k.match(key)
-            if m is not None:
-                ens = ensurer
+            if k.match(key) is not None:
                 break
         else:
-            ens = default
-        self.ensurers[key] = ens
-        return ens
+            ensurer = default
+        self.ensurers[key] = ensurer
+        return ensurer
 
     def get_docs(self, key, default=VarDocs('<no documentation>')):
         """Gets the documentation for the environment variable."""
@@ -567,16 +587,27 @@ class Env(MutableMapping):
         return vd
 
     @contextmanager
-    def swap(self, other):
+    def swap(self, other=None, **kwargs):
         """Provides a context manager for temporarily swapping out certain
         environment variables with other values. On exit from the context
         manager, the original values are restored.
         """
         old = {}
-        for k, v in other.items():
+
+        # single positional argument should be a dict-like object
+        if other is not None:
+            for k, v in other.items():
+                old[k] = self.get(k, NotImplemented)
+                self[k] = v
+
+        # kwargs could also have been sent in
+        for k, v in kwargs.items():
             old[k] = self.get(k, NotImplemented)
             self[k] = v
+
         yield self
+
+        # restore the values
         for k, v in old.items():
             if v is NotImplemented:
                 del self[k]
@@ -589,6 +620,8 @@ class Env(MutableMapping):
     #
 
     def __getitem__(self, key):
+        if key is Ellipsis:
+            return self
         m = self._arg_regex.match(key)
         if (m is not None) and (key not in self._d) and ('ARGS' in self._d):
             args = self._d['ARGS']
@@ -615,11 +648,20 @@ class Env(MutableMapping):
         if not ensurer.validate(val):
             val = ensurer.convert(val)
         self._d[key] = val
-        self._detyped = None
+        if self.detypeable(val):
+            self._detyped = None
+            if self.get('UPDATE_OS_ENVIRON'):
+                if self._orig_env is None:
+                    self.replace_env()
+                else:
+                    os.environ[key] = ensurer.detype(val)
 
     def __delitem__(self, key):
-        del self._d[key]
-        self._detyped = None
+        val = self._d.pop(key)
+        if self.detypeable(val):
+            self._detyped = None
+            if self.get('UPDATE_OS_ENVIRON') and key in os.environ:
+                del os.environ[key]
 
     def get(self, key, default=None):
         """The environment will look up default values from its own defaults if a
@@ -685,15 +727,28 @@ def _is_executable_file(path):
 
 
 def yield_executables_windows(directory, name):
+    normalized_name = os.path.normcase(name)
     extensions = builtins.__xonsh_env__.get('PATHEXT')
-    for a_file in os.listdir(directory):
-        base_name, ext = os.path.splitext(a_file)
-        if name == base_name and ext.upper() in extensions:
+    try:
+        names = os.listdir(directory)
+    except PermissionError:
+        return
+    for a_file in names:
+        normalized_file_name = os.path.normcase(a_file)
+        base_name, ext = os.path.splitext(normalized_file_name)
+
+        if (
+            normalized_name == base_name or normalized_name == normalized_file_name
+        ) and ext.upper() in extensions:
             yield os.path.join(directory, a_file)
 
 
 def yield_executables_posix(directory, name):
-    if name in os.listdir(directory):
+    try:
+        names = os.listdir(directory)
+    except PermissionError:
+        return
+    if name in names:
         path = os.path.join(directory, name)
         if _is_executable_file(path):
             yield path
@@ -706,9 +761,15 @@ def locate_binary(name):
     if os.path.isfile(name) and name != os.path.basename(name):
         return name
 
+    directories = builtins.__xonsh_env__.get('PATH')
+
+    # Windows users expect t obe able to execute files in the same directory without `./`
+    if ON_WINDOWS:
+        directories = [_get_cwd()] + directories
+
     try:
         return next(chain.from_iterable(yield_executables(directory, name) for
-                    directory in builtins.__xonsh_env__.get('PATH') if os.path.isdir(directory)))
+                    directory in directories if os.path.isdir(directory)))
     except StopIteration:
         return None
 
@@ -847,7 +908,7 @@ def get_hg_branch(cwd=None, root=None):
             with open(branch_path, 'r') as branch_file:
                 branch = branch_file.read()
         else:
-            branch = call_hg_command(['branch'], cwd)
+            branch = 'default'
 
         if os.path.exists(bookmark_path):
             with open(bookmark_path, 'r') as bookmark_file:
@@ -877,7 +938,7 @@ def git_dirty_working_directory(cwd=None, include_untracked=False):
     try:
         cmd = ['git', 'status', '--porcelain']
         if include_untracked:
-            cmd.append('--untracked-files=yes')
+            cmd.append('--untracked-files=normal')
         else:
             cmd.append('--untracked-files=no')
         s = subprocess.check_output(cmd,
@@ -964,11 +1025,14 @@ def _current_job():
 
 
 def env_name(pre_chars='(', post_chars=') '):
-    """Extract the current environment name from $VIRTUAL_ENV."""
-    env_path = __xonsh_env__.get('VIRTUAL_ENV', '')
-
+    """Extract the current environment name from $VIRTUAL_ENV or
+    $CONDA_DEFAULT_ENV if that is set
+    """
+    env_path = builtins.__xonsh_env__.get('VIRTUAL_ENV', '')
+    if len(env_path) == 0 and ON_ANACONDA:
+        pre_chars, post_chars = '[', '] '
+        env_path = builtins.__xonsh_env__.get('CONDA_DEFAULT_ENV', '')
     env_name = os.path.basename(env_path)
-
     return pre_chars + env_name + post_chars if env_name else ''
 
 
@@ -980,7 +1044,7 @@ else:
 
 FORMATTER_DICT = dict(
     user=os.environ.get(USER, '<user>'),
-    prompt_end='#' if IS_ROOT else '$',
+    prompt_end='#' if IS_SUPERUSER else '$',
     hostname=socket.gethostname().split('.', 1)[0],
     cwd=_replace_home_cwd,
     cwd_dir=lambda: os.path.dirname(_replace_home_cwd()),
@@ -992,6 +1056,7 @@ FORMATTER_DICT = dict(
     current_job=_current_job,
     env_name=env_name,
     )
+
 DEFAULT_VALUES['FORMATTER_DICT'] = dict(FORMATTER_DICT)
 
 _FORMATTER = string.Formatter()
@@ -1054,7 +1119,7 @@ def partial_format_prompt(template=DEFAULT_PROMPT, formatter_dict=None):
         if field is None:
             continue
         elif field.startswith('$'):
-            v = builtins.__xonsh_env__[name[1:]]
+            v = builtins.__xonsh_env__[name[1:]]  # FIXME `name` is an unresolved ref
             v = _FORMATTER.convert_field(v, conv)
             v = _FORMATTER.format_field(v, spec)
             toks.append(v)
@@ -1127,46 +1192,52 @@ def load_static_config(ctx, config=None):
                                 DEFAULT_VALUES.get('XONSH_ENCODING_ERRORS',
                                                    'surrogateescape'))
         with open(config, 'r', encoding=encoding, errors=errors) as f:
-            conf = json.load(f)
-        ctx['LOADED_CONFIG'] = True
+            try:
+                conf = json.load(f)
+                assert isinstance(conf, Mapping)
+                ctx['LOADED_CONFIG'] = True
+            except Exception as e:
+                conf = {}
+                ctx['LOADED_CONFIG'] = False
+                print_exception()
+                # JSONDecodeError was added in Python v3.5
+                jerr = json.JSONDecodeError \
+                       if hasattr(json, 'JSONDecodeError') else ValueError
+                if isinstance(e, jerr):
+                    msg = 'Xonsh config file is not valid JSON.'
+                else:
+                    msg = 'Could not load xonsh config.'
+                print(msg, file=sys.stderr)
     else:
         conf = {}
         ctx['LOADED_CONFIG'] = False
+    builtins.__xonsh_config__ = conf
     return conf
 
 
 def xonshrc_context(rcfiles=None, execer=None):
     """Attempts to read in xonshrc file, and return the contents."""
     loaded = builtins.__xonsh_env__['LOADED_RC_FILES'] = []
-    if (rcfiles is None or execer is None):
+    if rcfiles is None or execer is None:
         return {}
     env = {}
     for rcfile in rcfiles:
         if not os.path.isfile(rcfile):
             loaded.append(False)
             continue
-        with open(rcfile, 'r') as f:
-            rc = f.read()
-        if not rc.endswith('\n'):
-            rc += '\n'
-        fname = execer.filename
         try:
-            execer.filename = rcfile
-            execer.exec(rc, glbs=env)
+            run_script_with_cache(rcfile, execer, env)
             loaded.append(True)
         except SyntaxError as err:
             loaded.append(False)
             msg = 'syntax error in xonsh run control file {0!r}: {1!s}'
             warn(msg.format(rcfile, err), RuntimeWarning)
-        finally:
-            execer.filename = fname
+            continue
     return env
 
 
-def windows_env_fixes(ctx):
+def windows_foreign_env_fixes(ctx):
     """Environment fixes for Windows. Operates in-place."""
-    # Windows default prompt doesn't work.
-    ctx['PROMPT'] = DEFAULT_PROMPT
     # remove these bash variables which only cause problems.
     for ev in ['HOME', 'OLDPWD']:
         if ev in ctx:
@@ -1182,17 +1253,32 @@ def windows_env_fixes(ctx):
     ctx['PWD'] = _get_cwd()
 
 
-def default_env(env=None, config=None):
+def default_env(env=None, config=None, login=True):
     """Constructs a default xonsh environment."""
     # in order of increasing precedence
     ctx = dict(BASE_ENV)
     ctx.update(os.environ)
-    conf = load_static_config(ctx, config=config)
-    ctx.update(conf.get('env', ()))
-    ctx.update(load_foreign_envs(shells=conf.get('foreign_shells', DEFAULT_SHELLS),
-                                 issue_warning=False))
     if ON_WINDOWS:
-        windows_env_fixes(ctx)
+        # Windows style PROMPT definitions don't work in XONSH:
+        try:
+            del ctx['PROMPT']
+        except KeyError:
+            pass
+
+    if login:
+        conf = load_static_config(ctx, config=config)
+
+        foreign_env = load_foreign_envs(shells=conf.get('foreign_shells', DEFAULT_SHELLS),
+                                        issue_warning=False)
+        if ON_WINDOWS:
+            windows_foreign_env_fixes(foreign_env)
+
+        ctx.update(foreign_env)
+
+        # Do static config environment last, to allow user to override any of
+        # our environment choices
+        ctx.update(conf.get('env', ()))
+
     # finalize env
     if env is not None:
         ctx.update(env)
