@@ -8,12 +8,14 @@ import builtins
 import subprocess
 from warnings import warn
 from functools import lru_cache
+from tempfile import NamedTemporaryFile
 from collections import MutableMapping, Mapping, Sequence
 
 from xonsh.tools import to_bool, ensure_string
 
 
 COMMAND = """
+{seterrprevcmd}
 {prevcmd}
 echo __XONSH_ENV_BEG__
 {envcmd}
@@ -25,6 +27,7 @@ echo __XONSH_FUNCS_BEG__
 {funcscmd}
 echo __XONSH_FUNCS_END__
 {postcmd}
+{seterrpostcmd}
 """.strip()
 
 DEFAULT_BASH_FUNCSCMD = """
@@ -44,7 +47,7 @@ shopt -s extdebug
 namelocfilestr=$(declare -F $funcnames)
 shopt -u extdebug
 
-# print just names and files as JSON object 
+# print just names and files as JSON object
 read -r -a namelocfile <<< $namelocfilestr
 sep=" "
 namefile="{"
@@ -70,7 +73,7 @@ for name in ${(ok)functions}; do
   loc=${(z)loc}
   file=${loc[7,-1]}
   namefile="${namefile}\\"${name}\\":\\"${(Q)file:A}\\","
-done 
+done
 if [[ "{" == "${namefile}" ]]; then
   namefile="${namefile}}"
 else
@@ -79,40 +82,65 @@ fi
 echo ${namefile}
 """.strip()
 
+# mapping of shell name alises to keys in other lookup dictionaries.
+CANON_SHELL_NAMES = {
+    'bash': 'bash',
+    '/bin/bash': 'bash',
+    'zsh': 'zsh',
+    '/bin/zsh': 'zsh',
+    '/usr/bin/zsh': 'zsh',
+    'cmd': 'cmd',
+    'cmd.exe': 'cmd',
+}
+
 DEFAULT_ENVCMDS = {
     'bash': 'env',
-    '/bin/bash': 'env',
     'zsh': 'env',
-    '/bin/zsh': 'env',
-    '/usr/bin/zsh': 'env',
+    'cmd': 'set',
 }
 DEFAULT_ALIASCMDS = {
     'bash': 'alias',
-    '/bin/bash': 'alias',
     'zsh': 'alias -L',
-    '/bin/zsh': 'alias -L',
-    '/usr/bin/zsh': 'alias -L',
+    'cmd': '',
 }
 DEFAULT_FUNCSCMDS = {
     'bash': DEFAULT_BASH_FUNCSCMD,
-    '/bin/bash': DEFAULT_BASH_FUNCSCMD,
     'zsh': DEFAULT_ZSH_FUNCSCMD,
-    '/bin/zsh': DEFAULT_ZSH_FUNCSCMD,
-    '/usr/bin/zsh': DEFAULT_ZSH_FUNCSCMD,
+    'cmd': '',
 }
 DEFAULT_SOURCERS = {
     'bash': 'source',
-    '/bin/bash': 'source',
     'zsh': 'source',
-    '/bin/zsh': 'source',
-    '/usr/bin/zsh': 'source',
+    'cmd': 'call',
 }
+DEFAULT_TMPFILE_EXT = {
+    'bash': '.sh',
+    'zsh': '.zsh',
+    'cmd': '.bat',
+}
+DEFAULT_RUNCMD = {
+    'bash': '-c',
+    'zsh': '-c',
+    'cmd': '/C',
+}
+DEFAULT_SETERRPREVCMD = {
+    'bash': 'set -e',
+    'zsh': 'set -e',
+    'cmd': '@echo off',
+}
+DEFAULT_SETERRPOSTCMD = {
+    'bash': '',
+    'zsh': '',
+    'cmd': 'if errorlevel 1 exit 1',
+}
+
 
 @lru_cache()
 def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
                        aliascmd=None, extra_args=(), currenv=None,
                        safe=True, prevcmd='', postcmd='', funcscmd=None,
-                       sourcer=None):
+                       sourcer=None, use_tmpfile=False, tmpfile_ext=None,
+                       runcmd=None, seterrprevcmd=None, seterrpostcmd=None):
     """Extracts data from a foreign (non-xonsh) shells. Currently this gets
     the environment, aliases, and functions but may be extended in the future.
 
@@ -152,6 +180,23 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
         How to source a foreign shell file for purposes of calling functions
         in that shell. If this is None, a default value will attempt to be
         looked up based on the shell name.
+    use_tmpfile : bool, optional
+        This specifies if the commands are written to a tmp file or just
+        parsed directly to the shell
+    tmpfile_ext : str or None, optional
+        If tmpfile is True this sets specifies the extension used.
+    runcmd : str or None, optional
+        Command line switches to use when running the script, such as
+        -c for Bash and /C for cmd.exe.
+    seterrprevcmd : str or None, optional
+        Command that enables exit-on-error for the shell that is run at the
+        start of the script. For example, this is "set -e" in Bash. To disable
+        exit-on-error behavior, simply pass in an empty string.
+    seterrpostcmd : str or None, optional
+        Command that enables exit-on-error for the shell that is run at the end
+        of the script. For example, this is "if errorlevel 1 exit 1" in
+        cmd.exe. To disable exit-on-error behavior, simply pass in an
+        empty string.
 
     Returns
     -------
@@ -167,13 +212,31 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
         cmd.append('-i')
     if login:
         cmd.append('-l')
-    cmd.append('-c')
-    envcmd = DEFAULT_ENVCMDS.get(shell, 'env') if envcmd is None else envcmd
-    aliascmd = DEFAULT_ALIASCMDS.get(shell, 'alias') if aliascmd is None else aliascmd
-    funcscmd = DEFAULT_FUNCSCMDS.get(shell, 'echo {}') if funcscmd is None else funcscmd
+    shkey = CANON_SHELL_NAMES[shell]
+    envcmd = DEFAULT_ENVCMDS.get(shkey, 'env') if envcmd is None else envcmd
+    aliascmd = DEFAULT_ALIASCMDS.get(shkey, 'alias') if aliascmd is None else aliascmd
+    funcscmd = DEFAULT_FUNCSCMDS.get(shkey, 'echo {}') if funcscmd is None else funcscmd
+    tmpfile_ext = DEFAULT_TMPFILE_EXT.get(shkey, 'sh') if tmpfile_ext is None else tmpfile_ext
+    runcmd = DEFAULT_RUNCMD.get(shkey, '-c') if runcmd is None else runcmd
+    seterrprevcmd = DEFAULT_SETERRPREVCMD.get(shkey, '') \
+                        if seterrprevcmd is None else seterrprevcmd
+    seterrpostcmd = DEFAULT_SETERRPOSTCMD.get(shkey, '') \
+                        if seterrpostcmd is None else seterrpostcmd
     command = COMMAND.format(envcmd=envcmd, aliascmd=aliascmd, prevcmd=prevcmd,
-                             postcmd=postcmd, funcscmd=funcscmd).strip()
-    cmd.append(command)
+                             postcmd=postcmd, funcscmd=funcscmd,
+                             seterrprevcmd=seterrprevcmd,
+                             seterrpostcmd=seterrpostcmd).strip()
+
+    cmd.append(runcmd)
+
+    if not use_tmpfile:
+        cmd.append(command)
+    else:
+        tmpfile = NamedTemporaryFile(suffix=tmpfile_ext, delete=False)
+        tmpfile.write(command.encode('utf8'))
+        tmpfile.close()
+        cmd.append(tmpfile.name)
+
     if currenv is None and hasattr(builtins, '__xonsh_env__'):
         currenv = builtins.__xonsh_env__.detype()
     elif currenv is not None:
@@ -186,7 +249,10 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
     except (subprocess.CalledProcessError, FileNotFoundError):
         if not safe:
             raise
-        return {}, {}
+        return None, None
+    finally:
+        if use_tmpfile:
+            os.remove(tmpfile.name)
     env = parse_env(s)
     aliases = parse_aliases(s)
     funcs = parse_funcs(s, shell=shell, sourcer=sourcer)
@@ -196,6 +262,7 @@ def foreign_shell_data(shell, interactive=True, login=False, envcmd=None,
 
 ENV_RE = re.compile('__XONSH_ENV_BEG__\n(.*)__XONSH_ENV_END__', flags=re.DOTALL)
 
+
 def parse_env(s):
     """Parses the environment portion of string into a dict."""
     m = ENV_RE.search(s)
@@ -210,13 +277,14 @@ def parse_env(s):
 ALIAS_RE = re.compile('__XONSH_ALIAS_BEG__\n(.*)__XONSH_ALIAS_END__',
                       flags=re.DOTALL)
 
+
 def parse_aliases(s):
     """Parses the aliases portion of string into a dict."""
     m = ALIAS_RE.search(s)
     if m is None:
         return {}
     g1 = m.group(1)
-    items = [line.split('=', 1) for line in g1.splitlines() if \
+    items = [line.split('=', 1) for line in g1.splitlines() if
              line.startswith('alias ') and '=' in line]
     aliases = {}
     for key, value in items:
@@ -236,9 +304,10 @@ def parse_aliases(s):
     return aliases
 
 
-FUNCS_RE = re.compile('__XONSH_FUNCS_BEG__\n(.*)__XONSH_FUNCS_END__',
+FUNCS_RE = re.compile('__XONSH_FUNCS_BEG__\n(.+)\n__XONSH_FUNCS_END__',
                       flags=re.DOTALL)
 
+
 def parse_funcs(s, shell, sourcer=None):
     """Parses the funcs portion of a string into a dict of callable foreign
     function wrappers.
@@ -379,6 +448,12 @@ def ensure_shell(shell):
     if 'sourcer' in shell_keys:
         shell['sourcer'] = None if shell['sourcer'] is None \
                                  else ensure_string(shell['sourcer'])
+    if 'seterrprevcmd' in shell_keys:
+        shell['seterrprevcmd'] = None if shell['seterrprevcmd'] is None \
+                                 else ensure_string(shell['seterrprevcmd'])
+    if 'seterrpostcmd' in shell_keys:
+        shell['seterrpostcmd'] = None if shell['seterrpostcmd'] is None \
+                                 else ensure_string(shell['seterrpostcmd'])
     return shell
 
 
@@ -390,17 +465,13 @@ def _get_shells(shells=None, config=None, issue_warning=True):
     elif shells is not None:
         pass
     else:
-        if config is None:
-            config = builtins.__xonsh_env__.get('XONSHCONFIG')
-        if os.path.isfile(config):
-            with open(config, 'r') as f:
-                conf = json.load(f)
-            shells = conf.get('foreign_shells', DEFAULT_SHELLS)
+        env = getattr(builtins, '__xonsh_env__', os.environ)
+        if env.get('LOADED_CONFIG', False):
+            conf = builtins.__xonsh_config__
         else:
-            if issue_warning:
-                msg = 'could not find xonsh config file ($XONSHCONFIG) at {0!r}'
-                warn(msg.format(config), RuntimeWarning)
-            shells = DEFAULT_SHELLS
+            from xonsh.environ import load_static_config
+            conf = load_static_config(env, config)
+        shells = conf.get('foreign_shells', DEFAULT_SHELLS)
     return shells
 
 
@@ -429,7 +500,8 @@ def load_foreign_envs(shells=None, config=None, issue_warning=True):
     for shell in shells:
         shell = ensure_shell(shell)
         shenv, _ = foreign_shell_data(**shell)
-        env.update(shenv)
+        if shenv:
+            env.update(shenv)
     return env
 
 
@@ -458,5 +530,6 @@ def load_foreign_aliases(shells=None, config=None, issue_warning=True):
     for shell in shells:
         shell = ensure_shell(shell)
         _, shaliases = foreign_shell_data(**shell)
-        aliases.update(shaliases)
+        if shaliases:
+            aliases.update(shaliases)
     return aliases
