@@ -3,14 +3,17 @@
 
 Written using a hybrid of ``tokenize`` and PLY.
 """
-import tokenize
 
 from io import BytesIO
 from keyword import kwlist
 
-from ply.lex import LexToken
+try:
+    from ply.lex import LexToken
+except ImportError:
+    from xonsh.ply.lex import LexToken
 
-from xonsh.tools import VER_3_5, VER_MAJOR_MINOR
+from xonsh.platform import PYTHON_VERSION_INFO
+import xonsh.tokenize as tokenize
 
 token_map = {}
 """
@@ -32,6 +35,7 @@ _op_map = {
     '~': 'TILDE', '^': 'XOR', '<<': 'LSHIFT', '>>': 'RSHIFT',
     '<': 'LT', '<=': 'LE', '>': 'GT', '>=': 'GE', '==': 'EQ',
     '!=': 'NE', '->': 'RARROW',
+    '&&': 'AND', '||': 'OR',
     # assignment operators
     '=': 'EQUALS', '+=': 'PLUSEQUAL', '-=': 'MINUSEQUAL',
     '*=': 'TIMESEQUAL', '@=': 'ATEQUAL', '/=': 'DIVEQUAL', '%=': 'MODEQUAL',
@@ -43,17 +47,18 @@ for (op, type) in _op_map.items():
     token_map[(tokenize.OP, op)] = type
 
 token_map[tokenize.STRING] = 'STRING'
+token_map[tokenize.REGEXPATH] = 'REGEXPATH'
 token_map[tokenize.NEWLINE] = 'NEWLINE'
 token_map[tokenize.INDENT] = 'INDENT'
 token_map[tokenize.DEDENT] = 'DEDENT'
-if VER_3_5 <= VER_MAJOR_MINOR:
+if PYTHON_VERSION_INFO >= (3, 5, 0):
     token_map[tokenize.ASYNC] = 'ASYNC'
     token_map[tokenize.AWAIT] = 'AWAIT'
 
 _REDIRECT_NAMES = frozenset({'out', 'err', 'all', 'o', 'e', 'a'})
 
 
-def handle_name(state, token, stream):
+def handle_name(state, token):
     """
     Function for handling name tokens
     """
@@ -65,18 +70,18 @@ def handle_name(state, token, stream):
         yield _new_token(typ, token.string, token.start)
     else:
         # subprocess mode
-        n = next(stream, None)
+        n = next(state['stream'], None)
         string = token.string
         if (n is not None and n.string in {'<', '>', '>>'} and
                 n.start == token.end and token.string in _REDIRECT_NAMES):
             # looks like a redirect to me!
             e = n.end
             string += n.string
-            n2 = next(stream, None)
+            n2 = next(state['stream'], None)
             if n2 is not None and n2.string == '&' and n2.start == n.end:
                 string += n2.string
                 e = n2.end
-                n2 = next(stream, None)
+                n2 = next(state['stream'], None)
             if n2 is not None:
                 if (n2.start == e and
                         (n2.type == tokenize.NUMBER or
@@ -88,36 +93,44 @@ def handle_name(state, token, stream):
                 else:
                     state['last'] = n
                     yield _new_token('IOREDIRECT', string, token.start)
-                    yield from handle_token(state, n2, stream)
+                    yield from handle_token(state, n2)
             else:
                 state['last'] = n
                 yield _new_token('IOREDIRECT', string, token.start)
+        elif token.string == 'and':
+            yield _new_token('AND', token.string, token.start)
+            if n is not None:
+                yield from handle_token(state, n)
+        elif token.string == 'or':
+            yield _new_token('OR', token.string, token.start)
+            if n is not None:
+                yield from handle_token(state, n)
         else:
             yield _new_token('NAME', token.string, token.start)
             if n is not None:
-                yield from handle_token(state, n, stream)
+                yield from handle_token(state, n)
 
 
 def _make_special_handler(token_type, extra_check=lambda x: True):
-    def inner_handler(state, token, stream):
+    def inner_handler(state, token):
         state['last'] = token
         if state['pymode'][-1][0]:
             yield _new_token(token_type, token.string, token.start)
         else:
             # subprocess mode
-            n = next(stream, None)
+            n = next(state['stream'], None)
             string = token.string
             if (n is not None and
                     n.string in {'<', '>', '>>'} and
                     n.start == token.end):
                 e = n.end
                 string += n.string
-                n2 = next(stream, None)
+                n2 = next(state['stream'], None)
                 if n2 is not None and n2.string == '&' and n2.start == n.end:
                     state['last'] = n2
                     string += n2.string
                     e = n2.end
-                    n2 = next(stream, None)
+                    n2 = next(state['stream'], None)
                 if n2 is not None:
                     if (n2.start == e and
                             (n2.type == tokenize.NUMBER or
@@ -129,29 +142,54 @@ def _make_special_handler(token_type, extra_check=lambda x: True):
                     else:
                         state['last'] = n
                         yield _new_token('IOREDIRECT', string, token.start)
-                        yield from handle_token(state, n2, stream)
+                        yield from handle_token(state, n2)
                 else:
                     state['last'] = n
                     yield _new_token('IOREDIRECT', string, token.start)
             else:
                 yield _new_token(token_type, token.string, token.start)
                 if n is not None:
-                    yield from handle_token(state, n, stream)
+                    yield from handle_token(state, n)
     return inner_handler
 
 
 handle_number = _make_special_handler('NUMBER')
 """Function for handling number tokens"""
 
-handle_ampersand = _make_special_handler('AMPERSAND')
-"""Function for handling ampersand tokens"""
+def handle_ampersands(state, token):
+    """Function for generating PLY tokens for single and double ampersands."""
+    n = next(state['stream'], None)
+    if n is not None and n.type == tokenize.OP and \
+            n.string == '&' and n.start == token.end:
+        state['last'] = n
+        yield _new_token('AND', 'and', token.start)
+    else:
+        state['last'] = token
+        if state['pymode'][-1][0]:
+            yield _new_token('AMPERSAND', token.string, token.start)
+        if n is not None:
+            yield from handle_token(state, n)
 
 
-def handle_dollar(state, token, stream):
+def handle_pipes(state, token):
+    """Function for generating PLY tokens for single and double pipes."""
+    n = next(state['stream'], None)
+    if n is not None and n.type == tokenize.OP and \
+            n.string == '|' and n.start == token.end:
+        state['last'] = n
+        yield _new_token('OR', 'or', token.start)
+    else:
+        state['last'] = token
+        yield _new_token('PIPE', token.string, token.start)
+        if n is not None:
+            yield from handle_token(state, n)
+
+
+def handle_dollar(state, token):
     """
     Function for generating PLY tokens associated with ``$``.
     """
-    n = next(stream, None)
+    n = next(state['stream'], None)
 
     if n is None:
         m = "missing token after $"
@@ -180,11 +218,11 @@ def handle_dollar(state, token, stream):
         yield _new_token("ERRORTOKEN", m, token.start)
 
 
-def handle_at(state, token, stream):
+def handle_at(state, token):
     """
     Function for generating PLY tokens associated with ``@``.
     """
-    n = next(stream, None)
+    n = next(state['stream'], None)
 
     if n is None:
         state['last'] = token
@@ -198,14 +236,14 @@ def handle_at(state, token, stream):
     else:
         state['last'] = token
         yield _new_token('AT', '@', token.start)
-        yield from handle_token(state, n, stream)
+        yield from handle_token(state, n)
 
 
-def handle_question(state, token, stream):
+def handle_question(state, token):
     """
     Function for generating PLY tokens for help and superhelp
     """
-    n = next(stream, None)
+    n = next(state['stream'], None)
 
     if n is not None and n.type == tokenize.ERRORTOKEN and \
             n.string == '?' and n.start == token.end:
@@ -215,14 +253,14 @@ def handle_question(state, token, stream):
         state['last'] = token
         yield _new_token('QUESTION', '?', token.start)
         if n is not None:
-            yield from handle_token(state, n, stream)
+            yield from handle_token(state, n)
 
 
-def handle_bang(state, token, stream):
+def handle_bang(state, token):
     """
     Function for generating PLY tokens starting with !
     """
-    n = next(stream, None)
+    n = next(state['stream'], None)
     if (n is not None and n.type == tokenize.OP
             and n.string == '(' and n.start == token.end):
         state['pymode'].append((False, '!(', ')', token.start))
@@ -238,33 +276,7 @@ def handle_bang(state, token, stream):
         yield _new_token("ERRORTOKEN", e, token.start)
 
 
-def handle_backtick(state, token, stream):
-    """
-    Function for generating PLY tokens representing regex globs.
-    """
-    n = next(stream, None)
-
-    found_match = False
-    sofar = '`'
-    while n is not None:
-        sofar += n.string
-        if n.type == tokenize.ERRORTOKEN and n.string == '`':
-            found_match = True
-            break
-        elif n.type == tokenize.NEWLINE or n.type == tokenize.NL:
-            break
-        n = next(stream, None)
-    if found_match:
-        state['last'] = n
-        yield _new_token('REGEXPATH', sofar, token.start)
-    else:
-        state['last'] = token
-        e = "Could not find matching backtick for regex on line {0}"
-        m = e.format(token.start[0])
-        yield _new_token("ERRORTOKEN", m, token.start)
-
-
-def handle_lparen(state, token, stream):
+def handle_lparen(state, token):
     """
     Function for handling ``(``
     """
@@ -273,7 +285,7 @@ def handle_lparen(state, token, stream):
     yield _new_token('LPAREN', '(', token.start)
 
 
-def handle_lbrace(state, token, stream):
+def handle_lbrace(state, token):
     """
     Function for handling ``{``
     """
@@ -282,7 +294,7 @@ def handle_lbrace(state, token, stream):
     yield _new_token('LBRACE', '{', token.start)
 
 
-def handle_lbracket(state, token, stream):
+def handle_lbracket(state, token):
     """
     Function for handling ``[``
     """
@@ -304,7 +316,7 @@ def _end_delimiter(state, token):
         return 'Unmatched "{}" at line {}, column {}'.format(s, l, c)
 
 
-def handle_rparen(state, token, stream):
+def handle_rparen(state, token):
     """
     Function for handling ``)``
     """
@@ -316,7 +328,7 @@ def handle_rparen(state, token, stream):
         yield _new_token('ERRORTOKEN', e, token.start)
 
 
-def handle_rbrace(state, token, stream):
+def handle_rbrace(state, token):
     """
     Function for handling ``}``
     """
@@ -328,7 +340,7 @@ def handle_rbrace(state, token, stream):
         yield _new_token('ERRORTOKEN', e, token.start)
 
 
-def handle_rbracket(state, token, stream):
+def handle_rbracket(state, token):
     """
     Function for handling ``]``
     """
@@ -340,7 +352,7 @@ def handle_rbracket(state, token, stream):
         yield _new_token('ERRORTOKEN', e, token.start)
 
 
-def handle_error_space(state, token, stream):
+def handle_error_space(state, token):
     """
     Function for handling special whitespace characters in subprocess mode
     """
@@ -351,7 +363,7 @@ def handle_error_space(state, token, stream):
         yield from []
 
 
-def handle_error_token(state, token, stream):
+def handle_error_token(state, token):
     """
     Function for handling error tokens
     """
@@ -363,7 +375,7 @@ def handle_error_token(state, token, stream):
     yield _new_token(typ, token.string, token.start)
 
 
-def handle_ignore(state, token, stream):
+def handle_ignore(state, token):
     """
     Function for handling tokens that should be ignored
     """
@@ -378,7 +390,10 @@ special_handlers = {
     tokenize.NAME: handle_name,
     tokenize.NUMBER: handle_number,
     tokenize.ERRORTOKEN: handle_error_token,
-    (tokenize.OP, '&'): handle_ampersand,
+    (tokenize.OP, '|'): handle_pipes,
+    (tokenize.OP, '||'): handle_pipes,
+    (tokenize.OP, '&'): handle_ampersands,
+    (tokenize.OP, '&&'): handle_ampersands,
     (tokenize.OP, '@'): handle_at,
     (tokenize.OP, '('): handle_lparen,
     (tokenize.OP, ')'): handle_rparen,
@@ -387,7 +402,6 @@ special_handlers = {
     (tokenize.OP, '['): handle_lbracket,
     (tokenize.OP, ']'): handle_rbracket,
     (tokenize.ERRORTOKEN, '$'): handle_dollar,
-    (tokenize.ERRORTOKEN, '`'): handle_backtick,
     (tokenize.ERRORTOKEN, '?'): handle_question,
     (tokenize.ERRORTOKEN, '!'): handle_bang,
     (tokenize.ERRORTOKEN, ' '): handle_error_space,
@@ -399,7 +413,7 @@ functions may manipulate the Lexer's state.
 """
 
 
-def handle_token(state, token, stream):
+def handle_token(state, token):
     """
     General-purpose token handler.  Makes use of ``token_map`` or
     ``special_map`` to yield one or more PLY tokens from the given input.
@@ -410,11 +424,9 @@ def handle_token(state, token, stream):
     state :
         The current state of the lexer, including information about whether
         we are in Python mode or subprocess mode, which changes the lexer's
-        behavior
+        behavior.  Also includes the stream of tokens yet to be considered.
     token :
         The token (from ``tokenize``) currently under consideration
-    stream :
-        A generator from which more tokens can be grabbed if necessary
     """
     typ = token.type
     st = token.string
@@ -426,12 +438,12 @@ def handle_token(state, token, stream):
             if cur[0] == old[0] and cur[1] > old[1]:
                 yield _new_token('WS', token.line[old[1]:cur[1]], old)
     if (typ, st) in special_handlers:
-        yield from special_handlers[(typ, st)](state, token, stream)
+        yield from special_handlers[(typ, st)](state, token)
     elif (typ, st) in token_map:
         state['last'] = token
         yield _new_token(token_map[(typ, st)], st, token.start)
     elif typ in special_handlers:
-        yield from special_handlers[typ](state, token, stream)
+        yield from special_handlers[typ](state, token)
     elif typ in token_map:
         state['last'] = token
         yield _new_token(token_map[typ], st, token.start)
@@ -445,12 +457,13 @@ def get_tokens(s):
     Given a string containing xonsh code, generates a stream of relevant PLY
     tokens using ``handle_token``.
     """
-    tokstream = tokenize.tokenize(BytesIO(s.encode('utf-8')).readline)
-    state = {'indents': [0], 'pymode': [(True, '', '', (0, 0))], 'last': None}
+    state = {'indents': [0], 'last': None,
+             'pymode': [(True, '', '', (0, 0))],
+             'stream': tokenize.tokenize(BytesIO(s.encode('utf-8')).readline)}
     while True:
         try:
-            token = next(tokstream)
-            yield from handle_token(state, token, tokstream)
+            token = next(state['stream'])
+            yield from handle_token(state, token)
         except StopIteration:
             if len(state['pymode']) > 1:
                 pm, o, m, p = state['pymode'][-1]
