@@ -2,6 +2,7 @@
 """The main xonsh script."""
 import os
 import sys
+import enum
 import builtins
 from argparse import ArgumentParser, ArgumentTypeError
 from contextlib import contextmanager
@@ -12,12 +13,16 @@ except ImportError:
     setproctitle = None
 
 from xonsh import __version__
+from xonsh.environ import DEFAULT_VALUES
 from xonsh.shell import Shell
 from xonsh.pretty import pprint, pretty
+from xonsh.proc import HiddenCompletedCommand
 from xonsh.jobs import ignore_sigtstp
-from xonsh.tools import HAVE_PYGMENTS, setup_win_unicode_console, print_color, ON_WINDOWS
+from xonsh.tools import setup_win_unicode_console, print_color
+from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
+from xonsh.codecache import run_script_with_cache, run_code_with_cache
 
-if HAVE_PYGMENTS:
+if HAS_PYGMENTS:
     import pygments
     from xonsh import pyghooks
 
@@ -50,12 +55,12 @@ parser.add_argument('-c',
                     dest='command',
                     required=False,
                     default=None)
-parser.add_argument('-i',
+parser.add_argument('-i', '--interactive',
                     help='force running in interactive mode',
                     dest='force_interactive',
                     action='store_true',
                     default=False)
-parser.add_argument('-l',
+parser.add_argument('-l', '--login',
                     help='run as a login shell',
                     dest='login',
                     action='store_true',
@@ -70,6 +75,16 @@ parser.add_argument('--no-rc',
                     dest='norc',
                     action='store_true',
                     default=False)
+parser.add_argument('--no-script-cache',
+                    help="Do not cache scripts as they are run",
+                    dest='scriptcache',
+                    action='store_false',
+                    default=True)
+parser.add_argument('--cache-everything',
+                    help="Use a cache, even for interactive commands",
+                    dest='cacheall',
+                    action='store_true',
+                    default=False)
 parser.add_argument('-D',
                     dest='defines',
                     help='define an environment variable, in the form of '
@@ -103,9 +118,11 @@ def arg_undoers():
         '-h': (lambda args: setattr(args, 'help', False)),
         '-V': (lambda args: setattr(args, 'version', False)),
         '-c': (lambda args: setattr(args, 'command', None)),
-        '-i': (lambda args: setattr(args, 'force_interactive', Fals)),
+        '-i': (lambda args: setattr(args, 'force_interactive', False)),
         '-l': (lambda args: setattr(args, 'login', False)),
         '-c': (lambda args: setattr(args, 'command', None)),
+        '--no-script-cache': (lambda args: setattr(args, 'scriptcache', True)),
+        '--cache-everything': (lambda args: setattr(args, 'cacheall', False)),
         '--config-path': (lambda args: delattr(args, 'config_path')),
         '--no-rc': (lambda args: setattr(args, 'norc', False)),
         '-D': (lambda args: setattr(args, 'defines', None)),
@@ -113,8 +130,12 @@ def arg_undoers():
         }
     au['--help'] = au['-h']
     au['--version'] = au['-V']
+    au['--interactive'] = au['-i']
+    au['--login'] = au['-l']
+
     return au
 
+
 def undo_args(args):
     """Undoes missaligned args."""
     au = arg_undoers()
@@ -126,17 +147,26 @@ def undo_args(args):
                 if a.startswith(k):
                     au[k](args)
 
+
 def _pprint_displayhook(value):
-    if value is not None:
-        builtins._ = None  # Set '_' to None to avoid recursion
-        if HAVE_PYGMENTS:
-            s = pretty(value)  # color case
-            lexer = pyghooks.XonshLexer()
-            tokens = list(pygments.lex(s, lexer=lexer))
-            print_color(tokens)
-        else:
-            pprint(value)  # black & white case
-        builtins._ = value
+    if value is None or isinstance(value, HiddenCompletedCommand):
+        return
+    builtins._ = None  # Set '_' to None to avoid recursion
+    if HAS_PYGMENTS:
+        s = pretty(value)  # color case
+        lexer = pyghooks.XonshLexer()
+        tokens = list(pygments.lex(s, lexer=lexer))
+        print_color(tokens)
+    else:
+        pprint(value)  # black & white case
+    builtins._ = value
+
+
+class XonshMode(enum.Enum):
+    single_command = 0
+    script_from_file = 1
+    script_from_stdin = 2
+    interactive = 3
 
 
 def premain(argv=None):
@@ -156,19 +186,38 @@ def premain(argv=None):
         version = '/'.join(('xonsh', __version__)),
         print(version)
         exit()
-    shell_kwargs = {'shell_type': args.shell_type}
+    shell_kwargs = {'shell_type': args.shell_type or
+                                  DEFAULT_VALUES.get('SHELL_TYPE'),
+                    'completer': False,
+                    'login': False,
+                    'scriptcache': args.scriptcache,
+                    'cacheall': args.cacheall}
+    if args.login:
+        shell_kwargs['login'] = True
     if args.config_path is None:
         shell_kwargs['config'] = args.config_path
     if args.norc:
         shell_kwargs['rc'] = ()
     setattr(sys, 'displayhook', _pprint_displayhook)
-    shell = builtins.__xonsh_shell__ = Shell(**shell_kwargs)
+    if args.command is not None:
+        args.mode = XonshMode.single_command
+        shell_kwargs['shell_type'] = 'none'
+    elif args.file is not None:
+        args.mode = XonshMode.script_from_file
+        shell_kwargs['shell_type'] = 'none'
+    elif not sys.stdin.isatty() and not args.force_interactive:
+        args.mode = XonshMode.script_from_stdin
+        shell_kwargs['shell_type'] = 'none'
+    else:
+        args.mode = XonshMode.interactive
+        shell_kwargs['completer'] = True
+        shell_kwargs['login'] = True
     from xonsh import imphooks
+    shell = builtins.__xonsh_shell__ = Shell(**shell_kwargs)
     env = builtins.__xonsh_env__
+    env['XONSH_LOGIN'] = shell_kwargs['login']
     if args.defines is not None:
         env.update([x.split('=', 1) for x in args.defines])
-    if args.login:
-        env['XONSH_LOGIN'] = True
     env['XONSH_INTERACTIVE'] = False
     if ON_WINDOWS:
         setup_win_unicode_console(env.get('WIN_UNICODE_CONSOLE', True))
@@ -180,37 +229,29 @@ def main(argv=None):
     args = premain(argv)
     env = builtins.__xonsh_env__
     shell = builtins.__xonsh_shell__
-    if args.command is not None:
+    if args.mode == XonshMode.single_command:
         # run a single command and exit
-        shell.default(args.command)
-    elif args.file is not None:
+        run_code_with_cache(args.command, shell.execer, mode='single')
+    elif args.mode == XonshMode.script_from_file:
         # run a script contained in a file
         if os.path.isfile(args.file):
-            with open(args.file) as f:
-                code = f.read()
-            code = code if code.endswith('\n') else code + '\n'
             sys.argv = args.args
             env['ARGS'] = [args.file] + args.args
-            code = shell.execer.compile(code, mode='exec', glbs=shell.ctx,
-                                        filename=args.file)
-            shell.execer.exec(code, mode='exec', glbs=shell.ctx)
+            run_script_with_cache(args.file, shell.execer, glb=shell.ctx, loc=None, mode='exec')
         else:
             print('xonsh: {0}: No such file or directory.'.format(args.file))
-    elif not sys.stdin.isatty() and not args.force_interactive:
+    elif args.mode == XonshMode.script_from_stdin:
         # run a script given on stdin
         code = sys.stdin.read()
-        code = code if code.endswith('\n') else code + '\n'
-        code = shell.execer.compile(code, mode='exec', glbs=shell.ctx,
-                                    filename='<stdin>')
-        shell.execer.exec(code, mode='exec', glbs=shell.ctx)
+        run_code_with_cache(code, shell.execer, glb=shell.ctx, loc=None, mode='exec')
     else:
         # otherwise, enter the shell
         env['XONSH_INTERACTIVE'] = True
         ignore_sigtstp()
         if not env['LOADED_CONFIG'] and not any(env['LOADED_RC_FILES']):
             print('Could not find xonsh configuration or run control files.')
-            code = '$[xonfig wizard --confirm]'
-            shell.execer.exec(code, mode='single', glbs=shell.ctx)
+            from xonsh import xonfig  # lazy import
+            xonfig.main(['wizard', '--confirm'])
         shell.cmdloop()
     postmain(args)
 
@@ -233,6 +274,5 @@ def main_context(argv=None):
     postmain(args)
 
 
-
 if __name__ == '__main__':
     main()
