@@ -1,14 +1,15 @@
 # -*- coding: utf-8 -*-
 """Implements the base xonsh parser."""
-import os
-import sys
 from collections import Iterable, Sequence, Mapping
 
-from ply import yacc
+try:
+    from ply import yacc
+except ImportError:
+    from xonsh.ply import yacc
 
 from xonsh import ast
 from xonsh.lexer import Lexer, LexToken
-from xonsh.tools import VER_3_5_1, VER_FULL
+from xonsh.platform import PYTHON_VERSION_INFO
 
 
 class Location(object):
@@ -128,9 +129,12 @@ def xonsh_superhelp(x, lineno=None, col=None):
     return xonsh_call('__xonsh_superhelp__', [x], lineno=lineno, col=col)
 
 
-def xonsh_regexpath(x, lineno=None, col=None):
-    """Creates the AST node for calling the __xonsh_regexpath__() function."""
-    return xonsh_call('__xonsh_regexpath__', [x], lineno=lineno, col=col)
+def xonsh_regexpath(x, pymode=False, lineno=None, col=None):
+    """Creates the AST node for calling the __xonsh_regexpath__() function.
+    The pymode argument indicate if it is called from subproc or python mode"""
+    pymode = ast.NameConstant(value=pymode, lineno=lineno, col_offset=col)
+    return xonsh_call('__xonsh_regexpath__', args=[x, pymode], lineno=lineno,
+                      col=col)
 
 
 def load_ctx(x):
@@ -606,7 +610,7 @@ class BaseParser(object):
         """tfpdef : name_tok colon_test_opt"""
         p1 = p[1]
         kwargs = {'arg': p1.value, 'annotation': p[2]}
-        if VER_FULL >= VER_3_5_1:
+        if PYTHON_VERSION_INFO >= (3, 5, 1):
             kwargs.update({
                 'lineno': p1.lineno,
                 'col_offset': p1.lexpos,
@@ -732,7 +736,7 @@ class BaseParser(object):
         """vfpdef : name_tok"""
         p1 = p[1]
         kwargs = {'arg': p1.value, 'annotation': None}
-        if VER_FULL >= VER_3_5_1:
+        if PYTHON_VERSION_INFO >= (3, 5, 1):
             kwargs.update({
                 'lineno': p1.lineno,
                 'col_offset': p1.lexpos,
@@ -1722,7 +1726,8 @@ class BaseParser(object):
         """atom : REGEXPATH"""
         p1 = ast.Str(s=p[1].strip('`'), lineno=self.lineno,
                      col_offset=self.col)
-        p[0] = xonsh_regexpath(p1, lineno=self.lineno, col=self.col)
+        p[0] = xonsh_regexpath(p1, pymode=True, lineno=self.lineno,
+                               col=self.col)
 
     def p_atom_dname(self, p):
         """atom : DOLLAR_NAME"""
@@ -2216,7 +2221,8 @@ class BaseParser(object):
         """subproc_atom : REGEXPATH"""
         p1 = ast.Str(s=p[1].strip('`'), lineno=self.lineno,
                      col_offset=self.col)
-        p0 = xonsh_regexpath(p1, lineno=self.lineno, col=self.col)
+        p0 = xonsh_regexpath(p1, pymode=False, lineno=self.lineno,
+                             col=self.col)
         p0._cliarg_action = 'extend'
         p[0] = p0
 
