@@ -1,13 +1,10 @@
 # -*- coding: utf-8 -*-
 """History object for use with prompt_toolkit."""
-import os
 import time
 import builtins
 from threading import Thread
 
 import prompt_toolkit.history
-from prompt_toolkit.buffer import Buffer
-
 from xonsh import lazyjson
 
 
