@@ -17,72 +17,31 @@ Implementations:
 * indent()
 
 """
-import ctypes
 import os
 import re
 import sys
+import string
+import ctypes
 import builtins
-import platform
-import traceback
-import threading
 import subprocess
+import threading
+import traceback
+from warnings import warn
 from contextlib import contextmanager
 from collections import OrderedDict, Sequence, Set
-from warnings import warn
 
-#
-# Check pygments
-#
+# adding further imports from xonsh modules is discouraged to avoid cirular
+# dependencies
+from xonsh.platform import (has_prompt_toolkit, win_unicode_console,
+                            DEFAULT_ENCODING, ON_LINUX, ON_WINDOWS)
 
-def pygments_version():
-    """Returns the Pygments version or False."""
-    try:
-        import pygments
-        v = pygments.__version__
-    except ImportError:
-        v = False
-    return v
-
-if sys.version_info[0] >= 3:
-    string_types = (str, bytes)
-    unicode_type = str
+if has_prompt_toolkit():
+    import prompt_toolkit
 else:
-    string_types = (str, unicode)
-    unicode_type = unicode
-
-try:
-    import win_unicode_console
-except ImportError:
-    win_unicode_console = None
+    prompt_toolkit = None
 
 
-DEFAULT_ENCODING = sys.getdefaultencoding()
-
-ON_WINDOWS = (platform.system() == 'Windows')
-ON_MAC = (platform.system() == 'Darwin')
-ON_LINUX = (platform.system() == 'Linux')
-ON_ARCH = (platform.linux_distribution()[0] == 'arch')
-ON_POSIX = (os.name == 'posix')
-IS_ROOT = ctypes.windll.shell32.IsUserAnAdmin() != 0 if ON_WINDOWS else os.getuid() == 0
-HAVE_PYGMENTS = bool(pygments_version())
-
-VER_3_4 = (3, 4)
-VER_3_5 = (3, 5)
-VER_3_5_1 = (3, 5, 1)
-VER_FULL = sys.version_info[:3]
-VER_MAJOR_MINOR = sys.version_info[:2]
-V_MAJOR_MINOR = 'v{0}{1}'.format(*sys.version_info[:2])
-
-
-def docstring_by_version(**kwargs):
-    """Sets a docstring by the python version."""
-    doc = kwargs.get(V_MAJOR_MINOR, None)
-    if V_MAJOR_MINOR is None:
-        raise RuntimeError('unrecognized version ' + V_MAJOR_MINOR)
-    def dec(f):
-        f.__doc__ = doc
-        return f
-    return dec
+IS_SUPERUSER = ctypes.windll.shell32.IsUserAnAdmin() != 0 if ON_WINDOWS else os.getuid() == 0
 
 
 class XonshError(Exception):
@@ -95,10 +54,21 @@ class DefaultNotGivenType(object):
 
 DefaultNotGiven = DefaultNotGivenType()
 
+BEG_TOK_SKIPS = frozenset(['WS', 'INDENT', 'NOT', 'LPAREN'])
+END_TOK_TYPES = frozenset(['SEMI', 'AND', 'OR', 'RPAREN'])
+LPARENS = frozenset(['LPAREN', 'AT_LPAREN', 'BANG_LPAREN', 'DOLLAR_LPAREN'])
+
+def _is_not_lparen_and_rparen(lparens, rtok):
+    """Tests if an RPAREN token is matched with something other than a plain old
+    LPAREN type.
+    """
+    # note that any([]) is False, so this covers len(lparens) == 0
+    return rtok.type == 'RPAREN' and any(x != 'LPAREN' for x in lparens)
+
 
 def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
     """Excapsulates tokens in a source code line in a uncaptured
-    subprocess $[] starting at a minimum column. If there are no tokens
+    subprocess ![] starting at a minimum column. If there are no tokens
     (ie in a comment line) this returns None.
     """
     if lexer is None:
@@ -108,16 +78,23 @@ def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
     lexer.reset()
     lexer.input(line)
     toks = []
+    lparens = []
     end_offset = 0
     for tok in lexer:
         pos = tok.lexpos
-        if tok.type != 'SEMI' and pos >= maxcol:
+        if tok.type not in END_TOK_TYPES and pos >= maxcol:
             break
-        if len(toks) == 0 and tok.type in ('WS', 'INDENT'):
+        if tok.type in LPARENS:
+            lparens.append(tok.type)
+        if len(toks) == 0 and tok.type in BEG_TOK_SKIPS:
             continue  # handle indentation
-        elif len(toks) > 0 and toks[-1].type == 'SEMI':
-            if pos < maxcol and tok.type not in ('NEWLINE', 'DEDENT', 'WS'):
+        elif len(toks) > 0 and toks[-1].type in END_TOK_TYPES:
+            if _is_not_lparen_and_rparen(lparens, toks[-1]):
+                lparens.pop()  # don't continue or break
+            elif pos < maxcol and tok.type not in ('NEWLINE', 'DEDENT', 'WS'):
                 toks.clear()
+                if tok.type in BEG_TOK_SKIPS:
+                    continue
             else:
                 break
         if pos < mincol:
@@ -140,21 +117,25 @@ def subproc_toks(line, mincol=-1, maxcol=None, lexer=None, returnline=False):
                 tok.lexpos = len(line)
             break
     else:
+        if len(toks) > 0 and toks[-1].type in END_TOK_TYPES:
+            if _is_not_lparen_and_rparen(lparens, toks[-1]):
+                pass
+            else:
+                toks.pop()
         if len(toks) == 0:
             return  # handle comment lines
-        if toks[-1].type == 'SEMI':
-            toks.pop()
         tok = toks[-1]
         pos = tok.lexpos
-        if isinstance(tok.value, string_types):
-            end_offset = len(tok.value)
+        if isinstance(tok.value, str):
+            end_offset = len(tok.value.rstrip())
         else:
             el = line[pos:].split('#')[0].rstrip()
             end_offset = len(el)
     if len(toks) == 0:
         return  # handle comment lines
     beg, end = toks[0].lexpos, (toks[-1].lexpos + end_offset)
-    rtn = '$[' + line[beg:end] + ']'
+    end = len(line[:end].rstrip())
+    rtn = '![' + line[beg:end] + ']'
     if returnline:
         rtn = line[:beg] + rtn + line[end:]
     return rtn
@@ -365,10 +346,11 @@ def suggest_commands(cmd, env, aliases):
 
 def print_exception():
     """Print exceptions with/without traceback."""
-    if 'XONSH_SHOW_TRACEBACK' not in builtins.__xonsh_env__:
+    env = getattr(builtins, '__xonsh_env__', os.environ)
+    if 'XONSH_SHOW_TRACEBACK' not in env:
         sys.stderr.write('xonsh: For full traceback set: '
                          '$XONSH_SHOW_TRACEBACK = True\n')
-    if builtins.__xonsh_env__.get('XONSH_SHOW_TRACEBACK'):
+    if env.get('XONSH_SHOW_TRACEBACK', False):
         traceback.print_exc()
     else:
         exc_type, exc_value, exc_traceback = sys.exc_info()
@@ -410,17 +392,44 @@ def suggestion_sort_helper(x, y):
     return lendiff + inx + iny
 
 
-def escape_windows_title_string(s):
-    """Returns a string that is usable by the Windows cmd.exe title
-    builtin.  The escaping is based on details here and emperical testing:
+def escape_windows_cmd_string(s):
+    """Returns a string that is usable by the Windows cmd.exe.
+    The escaping is based on details here and emperical testing:
     http://www.robvanderwoude.com/escapechars.php
     """
-    for c in '^&<>|':
+    for c in '()%!^<>&|"':
         s = s.replace(c, '^' + c)
     s = s.replace('/?', '/.')
     return s
 
 
+def argvquote(arg, force=False):
+    """ Returns an argument quoted in such a way that that CommandLineToArgvW
+    on Windows will return the argument string unchanged.
+    This is the same thing Popen does when supplied with an list of arguments.
+    Arguments in a command line should be separated by spaces; this
+    function does not add these spaces. This implementation follows the
+    suggestions outlined here:
+    https://blogs.msdn.microsoft.com/twistylittlepassagesallalike/2011/04/23/everyone-quotes-command-line-arguments-the-wrong-way/
+    """
+    if not force and len(arg) != 0 and not any([c in arg for c in ' \t\n\v"']):
+        return arg
+    else:
+        n_backslashes = 0
+        cmdline = '"'
+        for c in arg:
+            if c == '"':
+                cmdline += (n_backslashes * 2 + 1) * '\\'
+            else:
+                cmdline += n_backslashes * '\\'
+            if c != '\\':
+                cmdline += c
+                n_backslashes = 0
+            else:
+                n_backslashes += 1
+        return cmdline + n_backslashes * 2 * '\\' + '"'
+
+
 def on_main_thread():
     """Checks if we are on the main thread or not."""
     return threading.current_thread() is threading.main_thread()
@@ -456,7 +465,7 @@ def is_float(x):
 
 def is_string(x):
     """Tests if something is a string"""
-    return isinstance(x, string_types)
+    return isinstance(x, str)
 
 
 def always_true(x):
@@ -471,19 +480,16 @@ def always_false(x):
 
 def ensure_string(x):
     """Returns a string if x is not a string, and x if it already is."""
-    if isinstance(x, string_types):
-        return x
-    else:
-        return str(x)
+    return str(x)
 
 
 def is_env_path(x):
     """This tests if something is an environment path, ie a list of strings."""
-    if isinstance(x, string_types):
+    if isinstance(x, str):
         return False
     else:
         return (isinstance(x, Sequence) and
-                all([isinstance(a, string_types) for a in x]))
+                all(isinstance(a, str) for a in x))
 
 
 def str_to_env_path(x):
@@ -509,7 +515,7 @@ def to_bool(x):
     """"Converts to a boolean in a semantically meaningful way."""
     if isinstance(x, bool):
         return x
-    elif isinstance(x, string_types):
+    elif isinstance(x, str):
         return False if x.lower() in _FALSES else True
     else:
         return bool(x)
@@ -522,8 +528,9 @@ def bool_to_str(x):
 
 _BREAKS = frozenset(['b', 'break', 's', 'skip', 'q', 'quit'])
 
+
 def to_bool_or_break(x):
-    if isinstance(x, string_types) and x.lower() in _BREAKS:
+    if isinstance(x, str) and x.lower() in _BREAKS:
         return 'break'
     else:
         return to_bool(x)
@@ -545,11 +552,8 @@ def ensure_int_or_slice(x):
 
 def is_string_set(x):
     """Tests if something is a set"""
-    if isinstance(x, string_types):
-        return False
-    else:
-        return (isinstance(x, Set) and
-                all([isinstance(a, string_types) for a in x]))
+    return (isinstance(x, Set) and
+            all(isinstance(a, str) for a in x))
 
 
 def csv_to_set(x):
@@ -567,14 +571,12 @@ def set_to_csv(x):
 
 def is_bool_seq(x):
     """Tests if an object is a sequence of bools."""
-    return isinstance(x, Sequence) and all(map(isinstance, x, [bool]*len(x)))
+    return isinstance(x, Sequence) and all(isinstance(y, bool) for y in x)
 
 
 def csv_to_bool_seq(x):
     """Takes a comma-separated string and converts it into a list of bools."""
-    if len(x) == 0:
-        return []
-    return list(map(to_bool, x.split(',')))
+    return [to_bool(y) for y in csv_to_set(x)]
 
 
 def bool_seq_to_csv(x):
@@ -592,8 +594,8 @@ def to_completions_display_value(x):
         x = 'none'
     elif x in {'multi', 'true'}:
         x = 'multi'
-    elif x in {'single'}:
-        x = 'single'
+    elif x == 'single':
+        pass
     else:
         warn('"{}" is not a valid value for $COMPLETIONS_DISPLAY. '.format(x) +
              'Using "multi".', RuntimeWarning)
@@ -693,7 +695,7 @@ def to_history_tuple(x):
     if not isinstance(x, (Sequence, float, int)):
         raise ValueError('history size must be given as a sequence or number')
     if isinstance(x, str):
-        m = RE_HISTORY_TUPLE.match(x.strip())
+        m = RE_HISTORY_TUPLE.match(x.strip().lower())
         return to_history_tuple((m.group(1), m.group(3)))
     elif isinstance(x, (float, int)):
         return to_history_tuple((x, 'commands'))
@@ -733,6 +735,72 @@ def color_style():
     return builtins.__xonsh_shell__.shell.color_style()
 
 
+def _get_color_indexes(style_map):
+    """ Generates the color and windows color index for a style """
+    table = prompt_toolkit.terminal.win32_output.ColorLookupTable()
+    pt_style = prompt_toolkit.styles.style_from_dict(style_map)
+    for token in style_map:
+        attr = pt_style.token_to_attrs[token]
+        if attr.color is not None:
+            index = table.lookup_color(attr.color, attr.bgcolor)
+            try:
+                rgb = (int(attr.color[0:2], 16),
+                       int(attr.color[2:4], 16),
+                       int(attr.color[4:6], 16))
+            except:
+                rgb = None
+            yield token, index, rgb
+
+
+def intensify_colors_for_cmd_exe(style_map, replace_colors=None):
+    """Returns a modified style to where colors that maps to dark
+       colors are replaced with brighter versions. Also expands the
+       range used by the gray colors
+    """
+    modified_style = {}
+    if not ON_WINDOWS or prompt_toolkit is None:
+        return modified_style
+    if replace_colors is None:
+        replace_colors = {1: '#44ffff',  # subst blue with bright cyan
+                          2: '#44ff44',  # subst green with bright green
+                          4: '#ff4444',  # subst red with bright red
+                          5: '#ff44ff',  # subst magenta with bright magenta
+                          6: '#ffff44',  # subst yellow with bright yellow
+                          9: '#00aaaa',  # subst intense blue (hard to read)
+                                         # with dark cyan (which is readable)
+                          }
+    for token, idx, _ in _get_color_indexes(style_map):
+        if idx in replace_colors:
+            modified_style[token] = replace_colors[idx]
+    return modified_style
+
+
+def expand_gray_colors_for_cmd_exe(style_map):
+    """ Expand the style's gray scale color range.
+        All gray scale colors has a tendency to map to the same default GRAY
+        in cmd.exe.
+    """
+    modified_style = {}
+    if not ON_WINDOWS or prompt_toolkit is None:
+        return modified_style
+    for token, idx, rgb in _get_color_indexes(style_map):
+        if idx == 7 and rgb:
+            if sum(rgb) <= 306:
+                # Equal and below '#666666 is reset to dark gray
+                modified_style[token] = '#444444'
+            elif sum(rgb) >= 408:
+                # Equal and above 0x888888 is reset to white
+                modified_style[token] = '#ffffff'
+    return modified_style
+
+
+def intensify_colors_on_win_setter(enable):
+    """ Resets the style when setting the INTENSIFY_COLORS_ON_WIN
+        environment variable. """
+    enable = to_bool(enable)
+    delattr(builtins.__xonsh_shell__.shell.styler, 'style_name')
+    return enable
+
 
 _RE_STRING_START = "[bBrRuU]*"
 _RE_STRING_TRIPLE_DOUBLE = '"""'
@@ -857,9 +925,8 @@ def expandvars(path):
     if isinstance(path, bytes):
         path = path.decode(encoding=ENV.get('XONSH_ENCODING'),
                            errors=ENV.get('XONSH_ENCODING_ERRORS'))
-    if '$' not in path and ((not ON_WINDOWS) or ('%' not in path)):
+    if '$' not in path and (not ON_WINDOWS or '%' not in path):
         return path
-    import string
     varchars = string.ascii_letters + string.digits + '_-'
     quote = '\''
     percent = '%'
@@ -917,6 +984,8 @@ def expandvars(path):
                         var = eval(var, builtins.__xonsh_ctx__)
                         if _is_in_env(var):
                             value = _get_env_string(var)
+                        elif var is Ellipsis:
+                            value = dollar + brace + '...' + rbrace
                         else:
                             value = dollar + brace + var + rbrace
                     except:
@@ -983,28 +1052,30 @@ class CommandsCache(Set):
 
     @property
     def all_commands(self):
-        path = builtins.__xonsh_env__.get('PATH', [])
+        paths = builtins.__xonsh_env__.get('PATH', [])
+        paths = frozenset(x for x in paths if os.path.isdir(x))
         # did PATH change?
-        path_hash = hash(frozenset(path))
+        path_hash = hash(paths)
         cache_valid = path_hash == self._path_checksum
         self._path_checksum = path_hash
         # did aliases change?
-        al_hash = hash(frozenset(builtins.aliases.keys()))
+        al_hash = hash(frozenset(builtins.aliases))
         cache_valid = cache_valid and al_hash == self._alias_checksum
         self._alias_checksum = al_hash
-        pm = self._path_mtime
         # did the contents of any directory in PATH change?
-        for d in filter(os.path.isdir, path):
-            m = os.stat(d).st_mtime
-            if m > pm:
-                pm = m
-                cache_valid = False
-        self._path_mtime = pm
+        max_mtime = 0
+        for path in paths:
+            mtime = os.stat(path).st_mtime
+            if mtime > max_mtime:
+                max_mtime = mtime
+        cache_valid = cache_valid and max_mtime > self._path_mtime
+        self._path_mtime = max_mtime
         if cache_valid:
             return self._cmds_cache
         allcmds = set()
-        for d in filter(os.path.isdir, path):
-            allcmds |= set(os.listdir(d))
-        allcmds |= set(builtins.aliases.keys())
+        for path in paths:
+            allcmds |= set(x for x in os.listdir(path)
+                           if os.path.isfile(x) and os.access(x, os.X_OK))
+            allcmds |= set(builtins.aliases)
         self._cmds_cache = frozenset(allcmds)
         return self._cmds_cache
